/*
 *	Title:		Escavalon - Interactive Fiction System
 *	Version:	Beta
 *	Created by:	Paolo Copello
 *	Home page:	http://web.tiscali.it/fanelia
 *	
 *	Copyright (C) 2003 Paolo Copello
 *	
 *	******************************************
 *	This file is part of Escavalon
 *	******************************************
 *	
 *	Escavalon is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *	
 */



// suppress "truncated debug info" warnings (VC++ specific)
#ifdef _MSC_VER
#pragma warning (disable: 4786)
#pragma warning (disable: 4788)
#endif // _MSC_VER

#include "Avalon.h"

#include "fnutils.h"
//#include "fnmaps.h"

// nota: in teoria quest'ultimo non dovrebbe essere incluso
#include "fnconsol.h"


/////////////////////////////////////
// AvDesigner
/////////////////////////////////////


AvDesigner::AvDesigner(	
			Fanel::IntegerMap<AvTerm>& terms,
			Fanel::StringSet& excludeWords,
			Fanel::IntegerMap<AvLocation>& locations,
			Fanel::IntegerMap<AvObject>& objects,
			Fanel::IntegerMap<AvIntVariable>& integerVariables,
			Fanel::IntegerMap<AvStrVariable>& stringVariables,
			Fanel::IntegerMap<AvTimer>& timers)

:	_terms(terms), _excludeWords(excludeWords), 
	_locations(locations), _objects(objects),
	_integerVariables(integerVariables), _stringVariables(stringVariables),
	_timers(timers)
{
}

void AvDesigner::AddTerm(int nID, const string& sWords)
{ 
	_terms.Add(nID, AvTerm(nID, sWords));
}

void AvDesigner::AddWordsToTerm(int nID, const string& sWords)
{
	_terms[nID].AddWords(sWords);
}

void AvDesigner::AddExcludeWords(const string& sWords)
{ 
	AvTerm tmp(1, sWords);
	_excludeWords.Add(tmp.GetWordList());
}

void AvDesigner::AddLocation(int nID, const string& sShortDesc, const string& sLongDesc,
		   int north, int south, int east, int west, int up, int down)
{ 
	_locations.Add(nID, AvLocation(nID, sShortDesc, sLongDesc, 
					   north, south, east, west, up, down) ) ;
}

void AvDesigner::AddStateToLocation(int nLocID, int nStateID, const string& sShortDesc, const string& sLongDesc)
{
	_locations[nLocID].AddState(AvState(nStateID, sShortDesc, sLongDesc));
}

void AvDesigner::AddObject(int nID, const string& sShortDesc, const string& sLongDesc,
	int nInitialLocation, bool bFixed, AvObject::OBJECT_TYPE type) 
{ 
	_objects.Add(nID, AvObject(nID, sShortDesc, sLongDesc, nInitialLocation, bFixed, type) ) ;
}

void AvDesigner::AddStateToObject(int nObjID, int nStateID, const string& sShortDesc, const string& sLongDesc)
{
	_objects[nObjID].AddState(AvState(nStateID, sShortDesc, sLongDesc));
}

void AvDesigner::AddIntegerVariable(int nID, int nInitialValue)
{
	AvIntVariable v;
	v.nVal = nInitialValue;
	_integerVariables.Add(nID, v);
}

void AvDesigner::AddStringVariable(int nID, const string& sInitialValue)
{
	AvStrVariable v;
	v.sVal = sInitialValue;
	_stringVariables.Add(nID, v);
}

void AvDesigner::AddTimer(int nID)
{
	_timers.Add(nID, AvTimer(nID));
}


/////////////////////////////////////
// AvalonGame
/////////////////////////////////////

AvalonGame::AvalonGame()
:	_gameStatus(_DESIGN), _nCurrentLocation(AV_INVALID), 
	_nPreviousLocation(AV_INVALID),
	_bTickTimersNextTime(true)
{
	_pIO = new ConsoleIOManager;
	_command.reserve(4);
}

AvalonGame::~AvalonGame()
{
	delete _pIO;
}


AvLocation& AvalonGame::GetLocation()
{ 
	return _locations[_nCurrentLocation]; 
}


void AvalonGame::Run()
{
	bool bWantContinue = false;

	for(int i = 0; i < 25; i++) Io().WriteLine();
	Io().Write("# Escavalon Beta - Interactive Fiction System\n\n\n");

	OnTitle();

	Initialize();

	// TODO !!!!
	// - verifica la coerenza delle stanze
	// - la coerenza degli oggetti dovrebbe essere verificata
	//   al momento della loro creazione
	// - verifica la coerenza della action map

	// a questo punto bisogna chiamare...

	// main loop

	do
	{
		ShowMenu();

		 _gameStatus = _INPROGRESS;
		while(_gameStatus == _INPROGRESS)
		{
			// todo computa eventuali timer scaduti
			// se ok chiama la OnBeforeLocationDesc
			//		 se ok mostra desc della stanza
			//				chiama la OnAfterLocationDesc
			//				richiedi input
			//				esamina input
			//				se corretto, esegui la dispatch su action map

			_bTickTimersNextTime = true;

			if(OnBeforeLocationDesc())
			{
				_commandStrings.clear();
				Io().WriteLine();
				// se e' la prima volta che vedo questa stanza, mostra la desc completa
				if(GetLocation().IsVisited() || GetLocation().GetLongDesc().empty())
				{
					Io().Write(GetLocation().GetShortDesc());
				}
				else
				{
					Io().Write(GetLocation().GetLongDesc());
				}
				GetLocation().SetVisited();
				Io().WriteLine();
				ShowObjectsInLocation();
				OnAfterLocationDesc();

				Io().Write("\nCosa devo fare?\n");
				Io().ReadTokenized(_commandStrings);
				Io().WriteLine();
				if(DecodeInput())
				{
					DispatchAction(_nCurrentLocation, _command);
					if(_gameStatus != _INPROGRESS)
						break;
					TickTimers();
				}
			}
		}

		switch(_gameStatus)
		{
			case _WIN : 
				OnWin();
				bWantContinue = false;
				break;

			case _LOSE :
				OnLose();
				bWantContinue = Io().ReadYesNo("Vuoi giocare ancora? (S/N) ");
				if(bWantContinue)
					RestoreInitialGameState();
				break;

			case _EXIT :
				Io().Write("Ciao!\n"); 
				bWantContinue = false;
				break;

			case _RESTART:
				RestoreInitialGameState();
				bWantContinue = true;
				break;

			default : 
				bWantContinue = false;
				break;
		}
	}
	while(bWantContinue);

	Io().WaitForEnter();
}


void AvalonGame::ShowMenu()
{
	bool bMenuActive = true;
	int nChoice = 0;

	do
	{
		bMenuActive = true;
		Io().Write("Menu principale:\n"
			"  1 - Inizia nuova partita.\n"
			"  2 - Carica partita salvata.\n"
			"  3 - Informazioni sulla licenza di Escavalon.\n"
			"  4 - Istruzioni del gioco.\n"
			"Scelta ");
		Io().Read(nChoice);
		Io().WriteLine();
		switch(nChoice)
		{
		case 1 : 
			OnStart(); 
			bMenuActive = false;
			break;
		case 2 : 
			if(LoadGameBool())
				bMenuActive = false;
			break;
		case 3 :
			ShowLicenseInfo();
			break;
		case 4 :
			ShowGameInstructions();
			break;
		default:
			Io().Write("Scelta non valida.\n");
		}
	}
	while(bMenuActive);
}

void AvalonGame::ShowLicenseInfo()
{
	Io().Write("Escavalon - Copyright (C) 2003 Paolo Copello\n");
	Io().Write("http://web.tiscali.it/fanelia \n\n");

	Io().Write(
		"Escavalon e' software libero: significa che e' possibile copiarlo e modificarlo in base alle proprie esigenze senza bisogno di chiedere il permesso all'autore.\n"
		"Se si vuole ridistribuire Escavalon, o una versione modificata, o un qualasiasi programma da esso derivato, cio' e' consentito a patto che vengano rispettate le condizioni poste dalla Licenza Pubblica Generale del Progetto GNU (GNU GPL).\n"
		"Una copia della licenza e' distribuita insieme a Escavalon (la versione italiana e' stata aggiunta per chiarezza, l'unica versione legalmente riconosciuta e' quella inglese).\n\n"
		"\n");
	Io().WaitForEnter();
	
	Io().Write(
		"Escavalon is free software; you can redistribute it and/or modify\n"
		"it under the terms of the GNU General Public License as published by\n"
		"the Free Software Foundation; either version 2 of the License, or\n"
		"any later version.\n\n"
		"This program is distributed in the hope that it will be useful,\n"
		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
		"GNU General Public License for more details.\n\n"
		"You should have received a copy of the GNU General Public License\n"
		"along with this program; if not, write to the Free Software\n"
		"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n");
	Io().WaitForEnter();
	DoNotTickTimerNextTime();
}

void AvalonGame::ShowGameInstructions()
{
	Io().Write(
		"Il gioco non fa differenza tra maiuscolo e minuscolo;\n"
		"NON usare lettere accentate.\n\n"
		"--- Movimento ---\n"
		"Sono supportate le seguenti direzioni:\n"
		"  NORD\n"
		"  SUD\n"
		"  EST\n"
		"  OVEST\n"
		"  ALTO\n"
		"  BASSO\n"
		"(in casi particolari potrebbe essere necessario il comando VAI\n"
		"seguito dal luogo dove si vuole andare)\n");
	Io().WaitForEnter();

	Io().Write(
		"--- Comandi ---\n"
		"Frasi (semplici) all'imperativo inizianti con un verbo\n\n"
		"Per ripetere la descrizione dettagliata del luogo:\n"
		"  GUARDA \n\n"
		"Esaminare un oggetto:\n"
		"  GUARDA LA CHIAVE\n\n"
		"Prendere/lasciare oggetti, vedere gli oggetti posseduti:\n"
		"  PRENDI IL BASTONE\n"
		"  POSA LA BOMBA\n"
		"  INVENTARIO\n\n"
		"Altri comandi bisogna scoprirli provando, a seconda delle situazioni. Es.:\n"
		"  APRI LA PORTA\n"
		"  PARLA CON IL PAZZO\n\n"
		"Se vuoi pronunciare un incantesimo o una parola chiave, \n"
		"basta scrivere la formula/parola in questione.\n");
	Io().WaitForEnter();

	Io().Write(
		"--- Comandi abbreviati ---\n"
		"Per i comandi piu' usati e' riconosciuta la lettera iniziale:\n\n"
		"Direzioni: N, S, E, O, A, B\n"
		"  G (= GUARDA)\n"
		"  I (= INVENTARIO)\n\n"
		"--- Gestione delle partite ---\n\n"
		"SALVA\n"
		"CARICA\n"
		"RICOMINCIA\n"
		"FINE\n\n"
		"Durante il gioco digita ISTRUZIONI per visualizzare questa schermata.\n");
	Io().WaitForEnter();

	Io().Write(
		"--- Consigli (mutuati da quelli di Enrico 'Erix' Colombini) ---\n\n"
		"- Disegnare una mappa provando tutte le direzioni in tutti i posti\n"
		"    (non dimenticare alto e basso!)\n"
		"- Guardare tutti gli oggetti.\n"
		"- Leggere attentamente descrizioni e risposte.\n"
		"- Salvare prima di provare qualcosa di potenzialmente rischioso.\n"
		"- Usare la fantasia.\n"
		"- Giocare con qualche amico, e' molto piu' divertente.\n"
		"- Se si resta bloccati, dormirci sopra.\n"
		"- Non scrivere all'autore per chiedere aiuto! (beh, dopo 3 mesi, al limite...)\n");

	Io().WaitForEnter();
	DoNotTickTimerNextTime();
}


void AvalonGame::MoveTo(int nLocationID)
{
	if(_locations.Contains(nLocationID))
	{
		_nPreviousLocation = _nCurrentLocation;
		_nCurrentLocation = nLocationID;
	}
	else
		throw invalid_argument(string("MoveTo: invalid location ") + Fanel::ConvertToString(nLocationID) ) ;
}

void AvalonGame::GetInventory(list<AvObject>& lst)
{
	list<AvObject> listAllObjects;
	_objects.GetAsList(listAllObjects);
	list<AvObject>::const_iterator it = listAllObjects.begin();
	for(; it!= listAllObjects.end(); ++it)
	{
		if (it->GetLocationID() == AV_INVENTORY)
			lst.push_back(*it);
	}
}


void AvalonGame::WinGame()
{
	_gameStatus = _WIN;
}

void AvalonGame::LoseGame()
{
	_gameStatus = _LOSE;
}

void AvalonGame::OnMoveNorth()
{
	MoveDirection(GetLocation().North());
}

void AvalonGame::OnMoveSouth()
{
	MoveDirection(GetLocation().South());
}

void AvalonGame::OnMoveEast()
{
	MoveDirection(GetLocation().East());
}

void AvalonGame::OnMoveWest()
{
	MoveDirection(GetLocation().West());
}

void AvalonGame::OnMoveUp()
{
	MoveDirection(GetLocation().Up());
}

void AvalonGame::OnMoveDown()
{
	MoveDirection(GetLocation().Down());
}

void AvalonGame::WasteTime()
{
	Io().Write("Agli ordini!\n");
	for(int i = 0; i < 10; i++)
	{
		Io().Wait(1);
		Io().Write(".....");
	}
	Io().WriteLine();
}

///////////////////////////

void AvalonGame::NoAction()
{
}


////////////////////////////
// AvalonGame : private functions
////////////////////////////

void AvalonGame::Initialize()
{
	AvDesigner design(_terms, _excludeWords, _locations, _objects, 
				_integerVariables, _stringVariables, _timers);
	OnInit(design);
	CreateDictionary();
	SaveInitialGameState();
}

// crea il dizionario di default

void AvalonGame::InitDefaultItaliano(AvDesigner& design)
{
	// DIZIONARIO
	design.AddTerm(VAI_NORD, "n nord");	// direzioni N S E O A B
	design.AddTerm(VAI_SUD, "s sud");
	design.AddTerm(VAI_EST, "e est");
	design.AddTerm(VAI_OVEST, "o ovest");
	design.AddTerm(VAI_ALTO, "a alto");
	design.AddTerm(VAI_BASSO, "b basso");
	design.AddTerm(VAI_VERSO, "vai");

	design.AddTerm(PRENDI, "prendi raccogli afferra");
	design.AddTerm(LASCIA, "lascia posa");
	design.AddTerm(GUARDA, "g guarda esamina");
	design.AddTerm(INVENTARIO, "i cosa inventario");

	design.AddTerm(CARICA, "carica riprendi");
	design.AddTerm(SALVA, "salva registra");
	design.AddTerm(FINE_GIOCO, "basta fine");
	design.AddTerm(RICOMINCIA, "ricomincia riparti");

	design.AddTerm(ISTRUZIONI, "istruzioni aiuto");

	design.AddTerm(DAI_COSA, "dai consegna");
	design.AddTerm(APRI, "apri spalanca");
	design.AddTerm(ASPETTA, "aspetta attendi");
	design.AddTerm(DORMI, "dormi riposa riposati");

	design.AddTerm(PARLA, "parla dialoga conversa interroga");
	design.AddTerm(SALTA, "salta saltella");
	design.AddTerm(GRIDA, "urla grida");
	design.AddTerm(CANTA, "canta intona canticchia");

	design.AddTerm(FUGGI, "scappa fuggi");
	design.AddTerm(CORRI, "corri ");
	design.AddTerm(PAROLACCIA, "cazzo merda");
	design.AddTerm(INSULTA, "insulta offendi");
	design.AddTerm(INSULTA_COMPUTER, "scemo stupido idiota stronzo bastardo pirla cretino coglione vaffanculo cazzone troia puttana merdoso frocio frocione deficiente");

	design.AddTerm(UCCIDI, "uccidi ammazza assassina");
	design.AddTerm(ATTACCA, "attacca colpisci picchia");
	design.AddTerm(MUORI, "muori ammazzati suicidati ucciditi sparati");	
	design.AddTerm(USA_COSA, "usa adopera");
	design.AddTerm(MANGIA, "mangia divora ingoia");
	design.AddTerm(BEVI, "bevi");
	design.AddTerm(SPOSTA, "sposta muovi");
	design.AddTerm(TIRA, "tira");
	design.AddTerm(PREMI, "premi spingi schiaccia");

	design.AddTerm(ENTRA, "entra");
	design.AddTerm(ESCI, "esci vattene");

	// escludi parole: articoli, preposizioni...
	design.AddExcludeWords("il lo i gli la le l un una uno");
	design.AddExcludeWords("di a da ad al all allo agli alla alle con su sul sull sulla sulle sugli in nel");

	// AZIONI
	// sempre dalla piu' prioritaria alla meno prioritaria

	// movimento
	ACTION_VERB (AV_ANY_PLACE, VAI_NORD, &AvalonGame::OnMoveNorth);
	ACTION_VERB (AV_ANY_PLACE, VAI_SUD, &AvalonGame::OnMoveSouth);
	ACTION_VERB (AV_ANY_PLACE, VAI_EST, &AvalonGame::OnMoveEast);
	ACTION_VERB (AV_ANY_PLACE, VAI_OVEST, &AvalonGame::OnMoveWest);
	ACTION_VERB (AV_ANY_PLACE, VAI_ALTO, &AvalonGame::OnMoveUp);
	ACTION_VERB (AV_ANY_PLACE, VAI_BASSO, &AvalonGame::OnMoveDown);

/*	ACTION_VERB_NOUN (AV_ANY_PLACE, VAI_VERSO, VAI_NORD, &AvalonGame::OnMoveNorth);
	ACTION_VERB_NOUN (AV_ANY_PLACE, VAI_VERSO, VAI_SUD, &AvalonGame::OnMoveSouth);
	ACTION_VERB_NOUN (AV_ANY_PLACE, VAI_VERSO, VAI_EST, &AvalonGame::OnMoveEast);
	ACTION_VERB_NOUN (AV_ANY_PLACE, VAI_VERSO, VAI_OVEST, &AvalonGame::OnMoveWest);
	ACTION_VERB_NOUN (AV_ANY_PLACE, VAI_VERSO, VAI_ALTO, &AvalonGame::OnMoveUp);
	ACTION_VERB_NOUN (AV_ANY_PLACE, VAI_VERSO, VAI_BASSO, &AvalonGame::OnMoveDown);*/

/*	ACTION_VERB_NOUN (AV_ANY_PLACE, CORRI, VAI_NORD, &AvalonGame::OnMoveNorth);
	ACTION_VERB_NOUN (AV_ANY_PLACE, CORRI, VAI_SUD, &AvalonGame::OnMoveSouth);
	ACTION_VERB_NOUN (AV_ANY_PLACE, CORRI, VAI_EST, &AvalonGame::OnMoveEast);
	ACTION_VERB_NOUN (AV_ANY_PLACE, CORRI, VAI_OVEST, &AvalonGame::OnMoveWest);
	ACTION_VERB_NOUN (AV_ANY_PLACE, CORRI, VAI_ALTO, &AvalonGame::OnMoveUp);
	ACTION_VERB_NOUN (AV_ANY_PLACE, CORRI, VAI_BASSO, &AvalonGame::OnMoveDown);*/

/*	ACTION_VERB_NOUN (AV_ANY_PLACE, FUGGI, VAI_NORD, &AvalonGame::OnMoveNorth);
	ACTION_VERB_NOUN (AV_ANY_PLACE, FUGGI, VAI_SUD, &AvalonGame::OnMoveSouth);
	ACTION_VERB_NOUN (AV_ANY_PLACE, FUGGI, VAI_EST, &AvalonGame::OnMoveEast);
	ACTION_VERB_NOUN (AV_ANY_PLACE, FUGGI, VAI_OVEST, &AvalonGame::OnMoveWest);
	ACTION_VERB_NOUN (AV_ANY_PLACE, FUGGI, VAI_ALTO, &AvalonGame::OnMoveUp);
	ACTION_VERB_NOUN (AV_ANY_PLACE, FUGGI, VAI_BASSO, &AvalonGame::OnMoveDown);*/

	ACTION_VERB (AV_ANY_PLACE, VAI_VERSO, "Dimmi la direzione: N, S, E, O, A, B");
	ACTION_VERB_NOUN (AV_ANY_PLACE, VAI_VERSO, AV_ANY_TERM, "Per spostarti da un luogo all'altro, basta dirmi la direzione: N, S, E, O, A, B");
	
	ACTION_VERB (AV_ANY_PLACE, ESCI, "Dimmi la direzione: N S E O A B.\nPer uscire dal gioco digita FINE.");	
	ACTION_VERB (AV_ANY_PLACE, ENTRA, "Dimmi la direzione: N S E O A B");	
	ACTION_VERB_NOUN (AV_ANY_PLACE, ESCI, AV_ANY_TERM, "Dimmi la direzione: N S E O A B");
	ACTION_VERB_NOUN (AV_ANY_PLACE, ENTRA, AV_ANY_TERM, "Dimmi la direzione: N S E O A B");

	// gestione di default degli oggetti
	ACTION_VERB (AV_ANY_PLACE, GUARDA, &AvalonGame::LookAround);
	ACTION_VERB_NOUN (AV_ANY_PLACE, GUARDA, AV_ANY_TERM, &AvalonGame::LookObject);
	ACTION_VERB (AV_ANY_PLACE, INVENTARIO, &AvalonGame::ShowInventory);

	ACTION_VERB (AV_ANY_PLACE, PRENDI, "Magari dimmi anche cosa vuoi prendere...");
	ACTION_VERB (AV_ANY_PLACE, LASCIA, "Dimmi di quale oggetto vuoi sbarazzarti.");
	ACTION_VERB_NOUN (AV_ANY_PLACE, PRENDI, AV_ANY_TERM, &AvalonGame::PickupObject);
	ACTION_VERB_NOUN (AV_ANY_PLACE, LASCIA, AV_ANY_TERM, &AvalonGame::DropdownObject);

	// gestione delle partite
	ACTION_VERB (AV_ANY_PLACE, SALVA, &AvalonGame::SaveGame);
	ACTION_VERB (AV_ANY_PLACE, CARICA, &AvalonGame::LoadGame);
	ACTION_VERB (AV_ANY_PLACE, FINE_GIOCO, &AvalonGame::ExitGame);
	ACTION_VERB (AV_ANY_PLACE, RICOMINCIA, &AvalonGame::RestartGame	);
	ACTION_VERB (AV_ANY_PLACE, ISTRUZIONI, &AvalonGame::ShowGameInstructions );
	
	// messaggi di default per i vari verbi
	ACTION_VERB (AV_ANY_PLACE, DAI_COSA, "Dimmi: 'dai <oggetto>', oppure 'dai <oggetto> a <qualcuno>'.");
	ACTION_VERB_NOUN (AV_ANY_PLACE, DAI_COSA, AV_ANY_TERM, "Non mi sembra una buona idea.");
	ACTION_THREE (AV_ANY_PLACE, DAI_COSA, AV_ANY_TERM, AV_ANY_TERM, "Non mi sembra una buona idea.");

	ACTION_VERB (AV_ANY_PLACE, APRI, "Devi dirmi anche che cosa vuoi aprire!!!");
	ACTION_VERB_NOUN (AV_ANY_PLACE, APRI, AV_ANY_TERM, "Ma ti sembra che si possa aprire?!?");
	ACTION_VERB (AV_ANY_PLACE, ASPETTA, &AvalonGame::WasteTime);
	ACTION_VERB (AV_ANY_PLACE, DORMI, "Ma ti sembra il momento di fare un pisolino?");

	ACTION_VERB (AV_ANY_PLACE, PARLA, "Parlare da soli e' il primo sintomo della pazzia.");
	ACTION_VERB_NOUN (AV_ANY_PLACE, PARLA, AV_ANY_TERM, &AvalonGame::DefaultTalk);

	ACTION_VERB (AV_ANY_PLACE, SALTA, "Salti sul posto due o tre volte, ma non succede niente.");
	ACTION_VERB (AV_ANY_PLACE, GRIDA, "Senti solo l'eco della tua voce.");
	ACTION_VERB (AV_ANY_PLACE, CANTA, "Non ci provare, sei stonato come una campana.");
	ACTION_VERB (AV_ANY_PLACE, FUGGI, "Paura, eh? Dimmi in che direzione vuoi scappare.");
	ACTION_VERB (AV_ANY_PLACE, CORRI, "Hai fretta? Dimmi in che direzione vuoi correre.");
	ACTION_VERB (AV_ANY_PLACE, PAROLACCIA, "Non dire parolacce, maleducato!");
	ACTION_VERB (AV_ANY_PLACE, INSULTA, "Non e' una bella cosa, ma... chi vuoi insultare?");
	ACTION_VERB_NOUN (AV_ANY_PLACE, INSULTA, AV_ANY_TERM, &AvalonGame::DefaultInsult);

	// reagisce agli insulti
	ACTION_VERB (AV_ANY_PLACE, INSULTA_COMPUTER, "A me?!? Come osi insultarmi?");
	ACTION_VERB_NOUN (AV_ANY_PLACE, INSULTA_COMPUTER, INSULTA_COMPUTER, "A me?!? Come osi insultarmi?");
	ACTION_VERB_NOUN (AV_ANY_PLACE, INSULTA_COMPUTER, PAROLACCIA, "A me?!? Come osi insultarmi?");

	ACTION_VERB (AV_ANY_PLACE, UCCIDI, "Vuoi uccidere qualcuno?!? Chi?");
	ACTION_VERB_NOUN (AV_ANY_PLACE, UCCIDI, AV_ANY_TERM, &AvalonGame::DefaultKill);

	ACTION_VERB (AV_ANY_PLACE, ATTACCA, "Chi? Che cosa? Insomma, spiegati!");
	ACTION_VERB_NOUN (AV_ANY_PLACE, ATTACCA, AV_ANY_TERM, &AvalonGame::DefaultKill);

	ACTION_VERB (AV_ANY_PLACE, MUORI, "Scherzi? Hai una missione da compiere.");
	ACTION_VERB (AV_ANY_PLACE, USA_COSA, "Usa cosa? Usa il cervello? Usa e getta? U.S.A.?");
	ACTION_VERB_NOUN (AV_ANY_PLACE, USA_COSA, AV_ANY_TERM, "Cosa ci devo fare? Sii piu' specifico.");
	ACTION_THREE (AV_ANY_PLACE, USA_COSA, AV_ANY_TERM, AV_ANY_TERM, "Non sembra funzionare.");
	ACTION_VERB (AV_ANY_PLACE, MANGIA, "Che cosa?");
	ACTION_VERB (AV_ANY_PLACE, BEVI, "Che cosa?");
	ACTION_VERB_NOUN (AV_ANY_PLACE, MANGIA, AV_ANY_TERM, "Non e' una cosa che si mangia.");
	ACTION_VERB_NOUN (AV_ANY_PLACE, BEVI, AV_ANY_TERM, "Non e' una cosa che si beve.");

	ACTION_VERB (AV_ANY_PLACE, SPOSTA, "Che cosa?");
	ACTION_VERB (AV_ANY_PLACE, TIRA, "Che cosa?");
	ACTION_VERB (AV_ANY_PLACE, PREMI, "Che cosa?");
	ACTION_VERB_NOUN (AV_ANY_PLACE, SPOSTA, AV_ANY_TERM, "Non si sposta.");
	ACTION_VERB_NOUN (AV_ANY_PLACE, TIRA, AV_ANY_TERM, "Non si sposta.");
	ACTION_VERB_NOUN (AV_ANY_PLACE, PREMI, AV_ANY_TERM, "Non si sposta.");

}

bool AvalonGame::DecodeInput()
{
	_command.clear();
	//_command.reserve(_commandStrings.size());
	if(_commandStrings.empty())
	{
		Io().Write("Eh?!?\n");
		return false;
	}
	// il primo elemento e' il verbo, lo considera in modo speciale
	if(!_dictionary.Contains(_commandStrings.front()))
	{
		Io().Write(string("Non conosco il verbo '") + _commandStrings.front() + "'.\n");
		return false;
	}
	_command.push_back(_dictionary[_commandStrings.front()]);
	// parte dal secondo, controlla anche le parole da escludere
	vector<string>::const_iterator it = _commandStrings.begin(); 
	++it;
	for (; it!=_commandStrings.end(); ++it)
	{
		if(_excludeWords.Contains(*it))
			continue;
		if(!_dictionary.Contains(*it))
		{
			Io().Write(string("Non conosco la parola '") + *it + "'.\n");
			return false;
		}
		// se il termine corrisponde ad un oggetto, verifica
		// che si trovi nella stanza o addosso al protagonista
		int nID = _dictionary[*it];
		if(_objects.Contains(nID) && _objects[nID].GetLocationID() != AV_INVENTORY
				&& _objects[nID].GetLocationID() != _nCurrentLocation)
		{
			Io().Write( string("") + (*it) + "? Qui non ne vedo.\n" );
			return false;
		}
		_command.push_back(_dictionary[*it]);
	}
	return true;
}

void AvalonGame::CreateDictionary()
{
	list<AvTerm> termsList;
	_terms.GetAsList(termsList);
	list<AvTerm>::const_iterator it = termsList.begin();
	for(;it!=termsList.end(); ++it)
	{
		const list<string>& wList = it->GetWordList();
		list<string>::const_iterator innerIt = wList.begin();
		for(;innerIt!=wList.end(); ++innerIt)
			_dictionary.Add(*innerIt, it->GetID());
	}
}


void AvalonGame::MoveDirection(int dir)
{
	if(dir != AV_INVALID)
	{
		MoveTo(dir);
	}
	else
	{
		Io().Write("Di qui non puoi andare.\n");
	}
}

void AvalonGame::ShowObjectsInLocation()
{
	list<AvObject> lst;
	_objects.GetAsList(lst);
	list<AvObject>::const_iterator it = lst.begin();
	for(;it!=lst.end(); ++it)
	{
		if( it->GetLocationID() == _nCurrentLocation)
		{
			Io().Write(string("Vedo ") + it->GetShortDesc() + ".\n");
		}
	}
}

void AvalonGame::DoNotTickTimerNextTime()
{
	_bTickTimersNextTime = false;
}

void AvalonGame::TickTimers()
{
	// fa il tick di tutti i timer. Se un timer e' scaduto, 
	// lancia l'azione. Se a seguito dell'azione il gioco 
	// si interrompe, interrompe il ticking

	if(_bTickTimersNextTime)
	{
		Fanel::IntegerMap<AvTimer>::iterator it = _timers.begin();
		for(; it!= _timers.end(); ++it)
		{
			if(it->Tick() == 0)
			{
				DispatchTimerElapsed(it->GetID());
				if(_gameStatus != _INPROGRESS)
					return;
			}
			/*else if (it->IsActive())
			{
				DispatchTimerTick(it->GetID());
			}*/
		}
	}
}

void AvalonGame::LookAround()
{
	const string& sDesc = GetLocation().GetLongDesc();
	if(!sDesc.empty())
	{
		Io().Write(sDesc);
		Io().WriteLine();
	}
	else
	{
		Io().Write("Non vedo niente di speciale.\n");
	}
}

void AvalonGame::LookObject()
{
	if(_objects.Contains(_command[1]))
	{
		const string& sDesc = _objects[ _command[1] ].GetLongDesc();
		if(!sDesc.empty())
		{
			Io().Write(sDesc);
			Io().WriteLine();
		}
		else
		{
			Io().Write("Non vedo niente di speciale.\n");
		}
	}
	else
	{
		Io().Write("Non ho capito. Cosa vuoi guardare?!?\n");
	}
}

void AvalonGame::PickupObject()
{
	if(_objects.Contains(_command[1]))
	{
		AvObject& theObj = _objects[_command[1]];
		if(theObj.GetLocationID() == AV_INVENTORY)
		{
			Io().Write("Ce l'hai gia'.\n");
		} 
		else if(theObj.IsFixed())
		{
			// non si puo' prendere: rispondi in modo variopinto
			switch(theObj.GetType())
			{
				case AvObject::PLANT :
					Io().Write("Lascia quella pianta li' dov'e'.\n");
					break;
				case AvObject::ANIMAL :
				case AvObject::PERSON :
					Io().Write("Non vuole farsi prendere.\n");
					break;
				default:
					Io().Write("Stai scherzando?\n");
			}
		}
		else
		{
			// ok, si puo' prendere
			theObj.MoveTo(AV_INVENTORY);
			Io().Write("Fatto.\n");
		}
	}
	else
	{
		Io().Write("Non ho capito. Cosa vuoi prendere?!?\n");
	}
}

void AvalonGame::DropdownObject()
{
	if(_objects.Contains(_command[1]))
	{
		AvObject& theObj = _objects[_command[1]];
		if(theObj.GetLocationID() != AV_INVENTORY)
		{
			Io().Write("Non l'hai con te.\n");
		} 
		else
		{
			// ok, si puo' posare
			theObj.MoveTo(_nCurrentLocation);
			Io().Write("Fatto.\n");
		}
	}
	else
	{
		Io().Write("Non ho capito. Cosa vuoi lasciare?!?\n");
	}
}

void AvalonGame::ShowInventory()
{
	Io().Write("Possiedi:\n");
	list<AvObject> lst;
	GetInventory(lst);
	if(lst.empty())
	{
		Io().Write("- Un bel nulla.\n");
	}
	else
	{
		list<AvObject>::const_iterator it = lst.begin();
		for(; it != lst.end(); ++it)
		{
			Io().Write(string("- ") + it->GetShortDesc() + "\n");
		}
	}
}

void AvalonGame::DefaultTalk()
{
	if(_objects.Contains(_command[1]))
	{
		AvObject& theObj = _objects[_command[1]];
		switch(theObj.GetType())
		{
			case AvObject::THING :
				Io().Write("Hai una bella fantasia, davvero!\n");
				break;
			case AvObject::PLANT :
				Io().Write("Adesso vuoi pure parlare con le piante?\n");
				break;
			case AvObject::ANIMAL :
				Io().Write("Quest'avventura non si intitola 'Il dottor Dolittle'.\n");
				break;
			case AvObject::PERSON :
				Io().Write("Non sembra voler parlare con te.\n");
				break;
			default:
				Io().Write("Stai scherzando?\n");
		}
	}
	else
	{
		Io().Write("Non ho capito. Con chi vuoi parlare?!?\n");
	}
}
	
void AvalonGame::DefaultInsult()
{
	if(_objects.Contains(_command[1]))
	{
		AvObject& theObj = _objects[_command[1]];
		switch(theObj.GetType())
		{
			case AvObject::THING :
				Io().Write("Non credo che gli oggetti si offendano se insultati.\n");
				break;
			case AvObject::PLANT :
				Io().Write("Non si insultano le piante!\n");
				break;
			case AvObject::ANIMAL :
				Io().Write("Povera bestia, che male ti ha fatto?\n");
				break;
			case AvObject::PERSON :
				Io().Write("Non mi sembra una buona idea.\n");
				break;
			default:
				Io().Write("Stai scherzando?\n");
		}
	}
	else
	{
		Io().Write("Non ho capito. Chi vuoi insultare?!?\n");
	}
}


void AvalonGame::DefaultKill()
{
	if(_objects.Contains(_command[1]))
	{
		AvObject& theObj = _objects[_command[1]];
		switch(theObj.GetType())
		{
			case AvObject::PLANT :
				Io().Write("Perche' vorresti fare del male ad una pianta innocente?\n");
				break;
			case AvObject::ANIMAL :
				Io().Write("Povera bestia, che male ti ha fatto?\n");
				break;
			case AvObject::PERSON :
				Io().Write("Che male ti ha fatto, scusa?\n");
				break;
			default:
				if(_command[0] == UCCIDI)
					Io().Write("Stai scherzando?\n");
				else
					Io().Write("Non e' una buona idea, credimi.\n");
		}
	}
	else
	{
		Io().Write("Non ho capito. Chi vuoi uccidere?!?\n");
	}
}


/////////////////////////////////////
// SAVE/LOAD GAME
/////////////////////////////////////

void AvalonGame::SaveGameToStream(ostream& out)
{
		out << _nCurrentLocation << endl;
		out << _nPreviousLocation << endl;
		out << _stringVariables;
		out << _integerVariables;
		out << _objects;
		out << _locations;
		out << _timers;
}

void AvalonGame::LoadGameFromStream(istream& in)
{
		in >> _nCurrentLocation;
		Fanel::ClearInput(in);
		in >> _nPreviousLocation;
		Fanel::ClearInput(in);

		in >> _stringVariables;
		in >>  _integerVariables;
		in >> _objects;
		in >> _locations;
		in >> _timers;
}

void AvalonGame::SaveGame()
{
	string sFileName;
	DoNotTickTimerNextTime();
	Io().ReadFileName(sFileName, false);
	ofstream out(sFileName.c_str());
	if(out.good())
	{
		SaveGameToStream(out);
	}
	else
	{
		Io().Write("Errore, impossibile salvare la partita.\n");
	}
}

void AvalonGame::LoadGame()
{
	bool bDummy = LoadGameBool();
}

bool AvalonGame::LoadGameBool()
{
	string sFileName;
	DoNotTickTimerNextTime();
	Io().ReadFileName(sFileName, true);
	try
	{
		ifstream in (sFileName.c_str());
		if(in.good())
		{
			LoadGameFromStream(in);
		}
		else
		{
			Io().Write("Impossibile caricare la partita.\n");
			return false;
		}
	}
	catch(...)
	{
		Io().Write("Errore nel caricamento: file non valido.\n");
		return false;
	}
	return true;
}

void AvalonGame::SaveInitialGameState()
{
	ostrstream out_stream;
	SaveGameToStream(out_stream);
	_sInitialGameState = out_stream.str();
	out_stream.freeze(0);
}

void AvalonGame::RestoreInitialGameState()
{
	istrstream in_stream(_sInitialGameState.c_str());
	LoadGameFromStream(in_stream);
}

void AvalonGame::ExitGame()
{
	if(Io().ReadYesNo("Sei sicuro di voler uscire? (S/N) : "))
	{
		_gameStatus = _EXIT;
	}
}

void AvalonGame::RestartGame()
{
	if(Io().ReadYesNo("Sei sicuro di voler ricominciare da capo? (S/N) : "))
	{
		_gameStatus = _RESTART;
	}
}


