/*
 *	Title:		Escavalon - Interactive Fiction System
 *	Version:	Beta
 *	Created by:	Paolo Copello
 *	Home page:	http://web.tiscali.it/fanelia
 *	
 *	Copyright (C) 2003 Paolo Copello
 *	
 *	******************************************
 *	This file is part of Escavalon
 *	******************************************
 *	
 *	Escavalon is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	any later version.
 *	
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *	
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *	
 */


#if !defined(FANEL_AVALON_MAINMODULE_H__INCLUDED_)
#define FANEL_AVALON_MAINMODULE_H__INCLUDED_


///////////////////////////////////////////////////////////////

#include <vector>

#include "AvClass.h"
#include "AvAction.h"
#include "fnioman.h"

///////////////////////////////////////////////////////////////


class AvDesigner
{
public:
	AvDesigner(	Fanel::IntegerMap<AvTerm>& terms,
				Fanel::StringSet& excludeWords,
				Fanel::IntegerMap<AvLocation>& locations,
				Fanel::IntegerMap<AvObject>& objects,
				Fanel::IntegerMap<AvIntVariable>& integerVariables,
				Fanel::IntegerMap<AvStrVariable>& stringVariables,
				Fanel::IntegerMap<AvTimer>& timers);

	void AddTerm(int nID, const string& sWords);
	void AddWordsToTerm(int nID, const string& sWords);
	void AddExcludeWords(const string& sWords);

	void AddLocation(int nID, const string& sShortDesc, const string& sLongDesc,
			   int north, int south, int east, int west, int up, int down);
	void AddStateToLocation(int nLocID, int nStateID, const string& sShortDesc, const string& sLongDesc);

	void AddObject(int nID, const string& sShortDesc, const string& sLongDesc,
		int nInitialLocation, bool bFixed, AvObject::OBJECT_TYPE type = AvObject::UNSPECIFIED);
	void AddStateToObject(int nObjID, int nStateID, const string& sShortDesc, const string& sLongDesc);

	void AddTimer(int nID);

	void AddIntegerVariable(int nID, int nInitialValue = 0);
	void AddStringVariable(int nID, const string& sInitialValue = string());

private:
	Fanel::IntegerMap<AvTerm>& _terms;
	Fanel::StringSet& _excludeWords;
	Fanel::IntegerMap<AvLocation>& _locations;
	Fanel::IntegerMap<AvObject>& _objects;

	Fanel::IntegerMap<AvIntVariable>& _integerVariables;
	Fanel::IntegerMap<AvStrVariable>& _stringVariables;
	Fanel::IntegerMap<AvTimer>& _timers;

};

///////////////////////////////////////////////////////////////

#define AVALON_ACTION_MAP_NOBASE(class_name)	\
private:	\
	AvActionHandler<class_name> __handler;	\
protected:	\
	virtual void DispatchAction(int nLocation, vector<int> command)	\
	{	\
		if(!__handler.Handle(*this, nLocation, command))	\
		{	\
			Io().Write("Non ho capito.\n");	\
		}	\
	}	\
	virtual void DispatchTimerElapsed(int nTimer)	\
	{	\
		__handler.HandleTimerElapsed(*this, nTimer);	\
	}

	/*virtual void DispatchTimerTick(int nTimer)	\
	{	\
		__handler.HandleTimerTick(*this, nTimer);	\
	}*/


#define AVALON_ACTION_MAP(class_name, base_class)	\
private:	\
	AvActionHandler<class_name> __handler;	\
protected:	\
	virtual void DispatchAction(int nLocation, vector<int> command)	\
	{	\
		if(!__handler.Handle(*this, nLocation, command))	\
		{	\
			base_class::DispatchAction(nLocation, command);	\
		}	\
	}	\
	virtual void DispatchTimerElapsed(int nTimer)	\
	{	\
		if(!__handler.HandleTimerElapsed(*this, nTimer))	\
		{	\
			base_class::DispatchTimerElapsed(nTimer);	\
		}	\
	}

	/*virtual void DispatchTimerTick(int nTimer)	\
	{	\
		if(!__handler.HandleTimerTick(*this, nTimer))	\
		{	\
			base_class::DispatchTimerTick(nTimer);	\
		}	\
	}*/



#define ACTION_VERB(location, verb, func)	\
	__handler.AddAction(location, verb, func )


#define ACTION_VERB_NOUN(location, verb, noun, func)	\
	__handler.AddAction(location, verb, noun, func)


#define ACTION_THREE(location, verb, noun1, noun2, func)	\
	__handler.AddAction(location, verb, noun1, noun2, func)


#define ACTION_TIMER_ELAPSED(timer, func)	\
	__handler.AddActionTimerElapsed(timer, func);

/*#define ACTION_TIMER_TICK(timer, func)	\
	__handler.AddActionTimerTick(timer, func);*/


#define NO_ACTION &AvalonGame::NoAction


// i numeri negativi non sono ammessi come id di nessun item
// i numeri da 1 a 399 sono riservati

class AvalonGame
{
public:

	enum TERMINI_BASE_ITALIANO
	{
		VAI_NORD = 1,		// direzioni N S E O A B
		VAI_SUD = 2,
		VAI_EST = 3,
		VAI_OVEST = 4,
		VAI_ALTO = 5,
		VAI_BASSO = 6,
		VAI_VERSO = 7,			// vai
		PRENDI = 8,			// prendi/lascia
		LASCIA = 9, 
		GUARDA = 10,		// guarda (G)
		INVENTARIO = 11,// cosa (I)

		CARICA = 12,
		SALVA = 13,
		FINE_GIOCO = 14,
		RICOMINCIA = 15,  // per ora non previsto

		DAI_COSA = 16,
		APRI = 17,
		ASPETTA = 18,
		DORMI = 19,
		PARLA = 20,
		SALTA = 21,
		GRIDA = 22,
		CANTA = 23,
		FUGGI = 24,
		PAROLACCIA = 25,
		INSULTA = 26,  //insulta, offendi...
		INSULTA_COMPUTER = 27, //scemo, stupido...
		UCCIDI = 28,
		ATTACCA = 29,
		MUORI = 30,
		USA_COSA = 31,
		CORRI = 32,
		MANGIA = 33,
		BEVI = 34,
		SPOSTA = 35,
		TIRA = 36,
		PREMI = 37,
		ESCI,
		ENTRA,
		ISTRUZIONI

	};


	AvalonGame();
	virtual ~AvalonGame();

	void Run();

	///////////////////////////////////
	// the input/output management system
	///////////////////////////////////

	IOManager& Io() { return *_pIO; }

protected:
	///////////////////////////////////
	// to be called by the derived class
	///////////////////////////////////

	// crea dizionario e azioni di default in intaliano
	void InitDefaultItaliano(AvDesigner& design);

	// inspect/change the game status
	AvLocation& Locations(int nID) { return _locations[nID]; }
	AvObject& Objects(int nID) { return _objects[nID]; }
	AvTimer& Timers(int nID) { return _timers[nID]; }

	int&	IntegerVariables(int nID) { return _integerVariables[nID].nVal; }
	string&	StringVariables(int nID) { return _stringVariables[nID].sVal; }

	int GetPreviousLocationID() const { return _nPreviousLocation; }
	int GetLocationID() const { return _nCurrentLocation; }
	AvLocation& GetLocation();
	void MoveTo(int nLocationID); 
	void GetInventory(list<AvObject>& lst);
	void WinGame();
	void LoseGame();

	void WasteTime();

	const vector<int>& GetCommand() const { return _command; }
	const vector<string>& GetCommandStrings() const { return _commandStrings; }

	///////////////////////////////////
	// to be defined or overriden in the derived class
	///////////////////////////////////

	virtual void OnTitle() {  }
	virtual void OnStart() {  } // = 0;
	virtual void OnInit(AvDesigner& design) = 0;
	virtual void OnWin() {  } // = 0;
	virtual void OnLose() {  } // = 0;
	virtual bool OnBeforeLocationDesc() { return true; }
	virtual void OnAfterLocationDesc() { }

	virtual void OnMoveNorth();
	virtual void OnMoveSouth();
	virtual void OnMoveEast();
	virtual void OnMoveWest();
	virtual void OnMoveUp();
	virtual void OnMoveDown();

	void PickupObject();
	void DropdownObject();

	void NoAction();

	void DoNotTickTimerNextTime();
private:
	void ShowMenu();
	void Initialize();
	void CreateDictionary();
	bool DecodeInput();
	void ShowObjectsInLocation();
	void TickTimers();

	void ShowLicenseInfo();
	void ShowGameInstructions();


	// gestione azioni
	void MoveDirection(int dir);
	void LookAround();
	void LookObject();
	void ShowInventory();
	void DefaultTalk();
	void DefaultInsult();
	void DefaultKill();

	void SaveGame();
	void LoadGame();
	bool LoadGameBool();

	void ExitGame();
	void RestartGame();

	void LoadGameFromStream(istream& in);
	void SaveGameToStream(ostream& out);

	void SaveInitialGameState();
	void RestoreInitialGameState();

	enum _GAME_STATUS { _DESIGN, _INPROGRESS, _EXIT, _RESTART, _WIN, _LOSE };

	Fanel::IntegerMap<AvTerm> _terms;
	Fanel::IntegerMap<AvLocation> _locations;
	Fanel::IntegerMap<AvObject> _objects;

	Fanel::StringSet _excludeWords;
	Fanel::StringMap<int> _dictionary;

	Fanel::IntegerMap<AvIntVariable> _integerVariables;
	Fanel::IntegerMap<AvStrVariable> _stringVariables;

	Fanel::IntegerMap<AvTimer> _timers;

	vector<int> _command;
	vector<string> _commandStrings;

	IOManager* _pIO;

	int _nCurrentLocation;
	int _nPreviousLocation;

	bool _bTickTimersNextTime;

	_GAME_STATUS _gameStatus;

	string _sInitialGameState;

	AVALON_ACTION_MAP_NOBASE(AvalonGame)
};



#endif

